@@ -27,9 +27,9 @@
  * Original copyright notice appears below
  *
  * $Id: pssl.cxx 19752 2008-03-15 15:30:24Z hfriederich $
- * $Revision: 20385 $
+ * $Revision: 25519 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2011-04-06 03:26:18 -0500 (Wed, 06 Apr 2011) $
  */
 
 /* crypto/buffer/bss_sock.c */
@@ -104,10 +104,9 @@ extern "C" {
 
 
 #ifdef _MSC_VER
-
-#pragma comment(lib, P_SSL_LIB1)
-#pragma comment(lib, P_SSL_LIB2)
-
+  #pragma comment(lib, P_SSL_LIB1)
+  #pragma comment(lib, P_SSL_LIB2)
+  #pragma message("SSL support (via OpenSSL) enabled")
 #endif
 
 
@@ -658,7 +657,7 @@ PSSLDiffieHellman::~PSSLDiffieHellman()
     DH_free(dh);
 }
 
-#if defined(__BEOS__) || (defined(__APPLE__) && P_MACOSX < 900) || defined(__OpenBSD__)
+#ifdef P_d2i_DHparams_bio_OLD
 // 2/21/04 Yuri Kiryanov - fix for compiler choke on BeOS for usage of
 // SSL function d2i_DHparams_bio below in PSSLDiffieHellman::Load
 // 5/26/06 Hannes Friederich - Mac OS X seems to need that fix too...
@@ -800,12 +799,12 @@ PSSLContext::PSSLContext(const void * sessionId, PINDEX idSize)
 void PSSLContext::Construct(Method method, const void * sessionId, PINDEX idSize)
 {
   // create the new SSL context
+#if OPENSSL_VERSION_NUMBER >= 0x00909000L
+  const
+#endif
   SSL_METHOD * meth;
 
   switch (method) {
-    case SSLv2:
-      meth = SSLv2_method();
-      break;
     case SSLv3:
       meth = SSLv3_method();
       break;
@@ -1281,59 +1280,11 @@ PBoolean PSSLChannel::OnOpen()
 }
 
 
-//////////////////////////////////////////////////////////////////////////
-//
-//  misc unused code
-//
+#else
 
-#if 0
-
-extern "C" {
-
-static verify_depth = 0;
-static verify_error = VERIFY_OK;
-
-// should be X509 * but we can just have them as char *. 
-int verify_callback(int ok, X509 * xs, X509 * xi, int depth, int error)
-{
-  char *s;
-
-  s = (char *)X509_NAME_oneline(X509_get_subject_name(xs));
-  if (s == NULL) {
-//    ERR_print_errors(bio_err);
-    return(0);
-  }
-  PError << "depth= " << depth << " " << (char *)s << endl;
-  free(s);
-  if (error == VERIFY_ERR_UNABLE_TO_GET_ISSUER) {
-    s=(char *)X509_NAME_oneline(X509_get_issuer_name(xs));
-    if (s == NULL) {
-      PError << "verify error" << endl;
-      //ERR_print_errors(bio_err);
-      return(0);
-    }
-    PError << "issuer = " << s << endl;
-    free(s);
-  }
-
-  if (!ok) {
-    PError << "verify error:num=" << error << " " <<
-    X509_cert_verify_error_string(error) << endl;
-    if (verify_depth <= depth) {
-      ok=1;
-      verify_error=VERIFY_OK;
-    } else {
-      ok=0;
-      verify_error=error;
-    }
-  }
-  PError << "verify return:" << ok << endl;
-  return(ok);
-}
-
-};
-
-#endif
+  #ifdef _MSC_VER
+    #pragma message("SSL support (via OpenSSL) DISABLED")
+  #endif
 
 #endif // P_SSL
 
