@@ -24,9 +24,9 @@
  * Contributor(s): Craig Southeren
  *                 Snark at GnomeMeeting
  *
- * $Revision: 22443 $
- * $Author: rjongbloed $
- * $Date: 2009-04-20 18:47:22 -0500 (Mon, 20 Apr 2009) $
+ * $Revision: 24849 $
+ * $Author: shorne $
+ * $Date: 2010-10-31 02:59:47 -0500 (Sun, 31 Oct 2010) $
  */
 
 #ifdef __GNUC__
@@ -155,10 +155,16 @@ PString PSoundChannel::GetDefaultDevice(Directions dir)
 #endif
 
   PStringArray devices = GetDeviceNames(dir);
-  if (devices.GetSize() > 0)
-    return devices[0];
 
-  return PString::Empty();
+  if (devices.GetSize() == 0)
+    return PString::Empty();
+
+  for (PINDEX i = 0; i < devices.GetSize(); ++i) {
+    if (!(devices[i] *= "NULL"))
+      return devices[i];
+  }
+
+  return devices[0];
 }
 
 
@@ -296,6 +302,10 @@ PBoolean PSoundChannel::Write(const void * buf, PINDEX len)
   return baseChannel != NULL && baseChannel->Write(buf, len);
 }
 
+PBoolean PSoundChannel::Write(const void * buf, PINDEX len, const void * /*mark*/)
+{
+  return Write(buf, len);
+}
 
 PINDEX PSoundChannel::GetLastWriteCount() const
 {
