@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23344 $
+ * $Revision: 25739 $
  * $Author: rjongbloed $
- * $Date: 2009-09-05 01:12:00 -0500 (Sat, 05 Sep 2009) $
+ * $Date: 2011-05-11 01:47:02 -0500 (Wed, 11 May 2011) $
  */
 
 #include <ptlib.h>
@@ -501,7 +501,7 @@ void PServiceProcess::Terminate()
   PSystemLog::SetTarget(NULL);
 
   // Now end the program
-  exit(terminationValue);
+  _exit(terminationValue);
 }
 
 void PServiceProcess::PXOnAsyncSignal(int sig)
